/*

    qmailalizer - a qmail-send log analysis program

    Copyright (C) 2001  Pichler Wolfgang  (wpichler@dialog-gruppe.at)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version, and provided that the above
    copyright and permission notice is included with all distributed
    copies of this or derived software.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA

    This software uses the gd graphics library, which is copyright by
    Quest Protein Database Center, Cold Spring Harbor Labs.  Please
    see the documentation supplied with the library for additional
    information and license terms, or visit www.boutell.com/gd/ for the
    most recent version of the library and supporting documentation.


*/

#include "config.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <libxml/parser.h>

Config::Config ()
{
  // berprft ob eine config Datei vorhanden ist, wenn nicht dann wird eine Default erstellt
  checkforconfig ();
  // Liest die config
  parsefile ();
}

Config::~Config ()
{
  // Die Ressource wieder freigeben
  
}

void Config::checkforconfig ()
{ int fd;
  xmlDocPtr doc;
  xmlNodePtr tmp;
   
  // berprft als erstes ob die config existiert und ob sie gelesen werden kann
  fd = open ("/etc/qmailalizer.conf",O_RDONLY);
  if (fd == -1) {
    printf ("Cannot find or read 'etc/qmailalizer.conf'. Trying to generate it.\n");
    doc = xmlNewDoc((xmlChar *) "1.0");
    doc->charset = XML_CHAR_ENCODING_8859_1;
    doc->children = xmlNewDocNode(doc, NULL,(xmlChar *) "CONFIG",NULL);
    xmlNewChild(doc->children,NULL,(xmlChar *)"wwwdir",(xmlChar *)"/www");
    xmlNewChild(doc->children,NULL,(xmlChar *)"logdir",(xmlChar *)"/var/log/qmail/qmail-send/");
    xmlNewChild(doc->children,NULL,(xmlChar *)"phpname",(xmlChar *)"domain.php");
    xmlNewChild(doc->children,NULL,(xmlChar *)"imgdir",(xmlChar *)"/www/images");
    xmlNewChild(doc->children,NULL,(xmlChar *)"xmlfile",(xmlChar *)"/www/results.xml");
    xmlNewChild(doc->children,NULL,(xmlChar *)"xslfile1",(xmlChar *)"standard.xsl");
    xmlNewChild(doc->children,NULL,(xmlChar *)"xslfile2",(xmlChar *)"domain.xsl");
    xmlNewChild(doc->children,NULL,(xmlChar *)"palettfile",(xmlChar *)"/www/images/base.png");
    xmlNewChild(doc->children,NULL,(xmlChar *)"servertype",(xmlChar *)"vmailmgr");
    tmp = xmlNewChild(doc->children,NULL,(xmlChar *)"virtualdomains",NULL);
    xmlNewChild(tmp,NULL,(xmlChar *)"domain",(xmlChar *)"yourdomain.com");
    xmlSaveFile ("/etc/qmailalizer.conf",doc);
    xmlFreeDoc (doc);
    printf ("Just created a new qmailalizer.conf file. Please edit it to fit your needs.\n");
    exit (0);
  }
}

void Config::parsefile ()
{ xmlDocPtr xmlconfig;
  xmlNode *aktNode;
  xmlNode *aktDomainNode;
  xmlNodePtr cur;
  xmlChar *temp;
  LDomains *aktDomain=NULL;
  LDomains *nDomain;
  xmlChar *name;
  xmlChar *dname;
  
  xmlconfig = xmlParseFile("/etc/qmailalizer.conf");
  if (xmlconfig != NULL) {
    cur = xmlDocGetRootElement(xmlconfig);  
    if (cur != NULL) {
      if (xmlStrcmp(cur->name, (const xmlChar *) "CONFIG")==0) {     
        aktNode = cur->xmlChildrenNode;
        while (aktNode != NULL) {
          if (xmlStrcmp(aktNode->name,(const xmlChar *)"wwwdir")==0)
	  {
            temp = aktNode->children->content;
	    wwwdir = (char *)malloc(strlen((char *)aktNode->children->content)+1);
	    strcpy (wwwdir,(char *)temp);
	  }
          else if (xmlStrcmp(aktNode->name,(const xmlChar *)"logdir") == 0) {
            temp = aktNode->children->content;
	    logdir = (char *)malloc(strlen((char *)aktNode->children->content)+1);
	    strcpy (logdir,(char *)temp);
	  }
          else if (xmlStrcmp(aktNode->name,(const xmlChar *)"phpname") == 0) {
            temp = aktNode->children->content;
	    phpname = (char *)malloc(strlen((char *)aktNode->children->content)+1);
	    strcpy (phpname,(char *)temp);
	  }
          else if (xmlStrcmp(aktNode->name,(const xmlChar *)"imgdir") == 0) {
            temp = aktNode->children->content;
	    imgdir = (char *)malloc(strlen((char *)aktNode->children->content)+1);
	    strcpy (imgdir,(char *)temp);
	  }
          else if (xmlStrcmp(aktNode->name,(const xmlChar *)"xmlfile") == 0) {
            temp = aktNode->children->content;
	    xmlfile = (char *)malloc(strlen((char *)aktNode->children->content)+1);
	    strcpy (xmlfile,(char *)temp);
	  }
          else if (xmlStrcmp(aktNode->name,(const xmlChar *)"loguser") == 0) {
            temp = aktNode->children->content;
	    loguser = (char *)malloc(strlen((char *)aktNode->children->content)+1);
	    strcpy (loguser,(char *)temp);
	  }
          else if (xmlStrcmp(aktNode->name,(const xmlChar *)"xslfile1") == 0) {
            temp = aktNode->children->content;
	    xslfile1 = (char *)malloc(strlen((char *)aktNode->children->content)+1);
	    strcpy (xslfile1,(char *)temp);
	  }
          else if (xmlStrcmp(aktNode->name,(const xmlChar *)"xslfile2") == 0) {
            temp = aktNode->children->content;
	    xslfile2 = (char *)malloc(strlen((char *)aktNode->children->content)+1);
	    strcpy (xslfile2,(char *)temp);
	  }
          else if (xmlStrcmp(aktNode->name,(const xmlChar *)"palettfile") == 0) {
            temp = aktNode->children->content;
	    pfile = (char *)malloc(strlen((char *)aktNode->children->content)+1);
	    strcpy (pfile,(char *)temp);
	  }
	  else if (xmlStrcmp(aktNode->name,(const xmlChar *)"virtualdomainsfile") == 0) 
	    {
              temp = aktNode->children->content;
   	      vdomains = (char *)malloc(strlen((char *)aktNode->children->content)+1);
	      strcpy (vdomains,(char *)temp);
	    }

          else if (xmlStrcmp(aktNode->name,(const xmlChar *)"virtualdomains") == 0) {
	    aktDomainNode = aktNode->xmlChildrenNode;
	    vdomains = NULL;
	    ldWurzel = NULL;
	    if (aktDomainNode != NULL) {
	      vdomains = NULL;
	      // Es sind die domains direkt eingetragen, also gleich laden
	      while (aktDomainNode != NULL) {
	        if (xmlStrcmp(aktDomainNode->name,(const xmlChar *)"domain") == 0) {
    	          nDomain = new LDomains;
	          nDomain->next = NULL;
		  name = (xmlChar *)malloc(strlen((char *)aktDomainNode->children->content)+1);
		  strcpy ((char *)name,(char *)aktDomainNode->children->content);
		  temp = xmlGetProp(aktDomainNode,(xmlChar *)"suffix");
		  if (temp != NULL) {
  		    dname = (xmlChar *)malloc(strlen((char *)temp)+1);
		    strcpy ((char *)dname,(char *)temp);
  	            nDomain->unixuser = (char *)dname;
		  } else {
	          	nDomain->unixuser = NULL;
		  }
	          nDomain->domain = (char *)name;
	          if (aktDomain == NULL) {
	            aktDomain = nDomain;
		    ldWurzel = nDomain;
	          }
	          else
	          {
	            aktDomain->next = nDomain;
		    aktDomain = nDomain;
	          }
		}
		aktDomainNode = aktDomainNode->next;
	      }
	    
	    }
	  }
          else if (xmlStrcmp(aktNode->name,(const xmlChar *)"servertype") == 0) {
            temp = aktNode->children->content;
	    servertype = (char *)malloc(strlen((char *)aktNode->children->content)+1);
	    strcpy (servertype,(char *)temp);
	  }
          aktNode = aktNode->next;
        }
      }
    }
    xmlFreeDoc (xmlconfig);
  }
  xmlCleanupParser();
}
